#include <Arduino.h>
#include <M5StickCPlus.h>
#include <SparkFun_GridEYE_Arduino_Library.h>
#include <Wire.h>
#define SHARP 8  //原始温度矩阵为8*8
#define INSERT 5 //插值数量 在原来矩阵两点之间插入点的个数

GridEYE grideye;

float pixels[SHARP][SHARP];                                               //用来存储8*8数据
float HDTemp[SHARP + INSERT * (SHARP - 1)][SHARP + INSERT * (SHARP - 1)]; //用来存储插值后的数据
float MinTemp, MaxTemp, a, b, c, d;

//从8*8的矩阵中寻找最高温度和最低温度
void findmaxtemp()
{
  MinTemp = MaxTemp = pixels[0][0];
  for (uint8_t i = 0; i < SHARP; i++)
    for (uint8_t j = 0; j < SHARP; j++)
    {
      if (pixels[i][j] > MaxTemp)
        MaxTemp = pixels[i][j];
      if (pixels[i][j] < MinTemp)
        MinTemp = pixels[i][j];
    }
}

float BilinearInterpolation(float q11, float q12, float q21, float q22, float x1, float x2, float y1, float y2, float x, float y)
{
  float x2x1, y2y1, x2x, y2y, yy1, xx1;
  x2x1 = x2 - x1;
  y2y1 = y2 - y1;
  x2x = x2 - x;
  y2y = y2 - y;
  yy1 = y - y1;
  xx1 = x - x1;
  return 1.0 / (x2x1 * y2y1) * (q11 * x2x * y2y + q21 * xx1 * y2y + q12 * x2x * yy1 + q22 * xx1 * yy1);
}

//插值算法 8*8点阵过于稀疏，使用插值方式 放大点阵
void zoommix()
{
  //取4个点的值
  float q11, q12, q21, q22;
  uint8_t x11, x22, y11, y22;
  for (uint8_t row = 0; row < SHARP; row++)
  { //遍历原始数据的每一个点
    for (uint8_t col = 0; col < SHARP; col++)
    {
      q11 = pixels[row][col];
      x11 = row * INSERT + row;
      y11 = col * INSERT + col;
      q12 = pixels[row][col + 1 == SHARP ? SHARP - 1 : col + 1];
      q21 = pixels[row + 1 == SHARP ? SHARP - 1 : row + 1][col];
      q22 = pixels[row + 1 == SHARP ? SHARP - 1 : row + 1][col + 1 == SHARP ? SHARP - 1 : col + 1];
      x22 = (row + 1) * INSERT + row + 1;
      y22 = (col + 1) * INSERT + col + 1;

      //插入值
      for (uint8_t xpos = 0; xpos <= INSERT; xpos++)
      {
        for (uint8_t ypos = 0; ypos <= INSERT; ypos++)
        {
          if (x11 + xpos < SHARP + INSERT * (SHARP - 1) && y11 + ypos < SHARP + INSERT * (SHARP - 1))
          {
            HDTemp[x11 + xpos][y11 + ypos] = BilinearInterpolation(q11, q12, q21, q22, x11, x22, y11, y22, x11 + xpos, y11 + ypos);
          }
        }
      }
    }
  }
}

void Getabcd()
{
  a = MinTemp + (MaxTemp - MinTemp) * 0.2121;
  b = MinTemp + (MaxTemp - MinTemp) * 0.3182;
  c = MinTemp + (MaxTemp - MinTemp) * 0.4242;
  d = MinTemp + (MaxTemp - MinTemp) * 0.8182;
}

//浮点数转颜色  伪彩色
uint16_t GetColor(float val)
{
  byte red = 0, green = 0, blue = 0;
  red = constrain(255.0 / (c - b) * val - ((b * 255.0) / (c - b)), 0, 255);
  if ((val > MinTemp) & (val < a))
  {
    green = constrain(255.0 / (a - MinTemp) * val - (255.0 * MinTemp) / (a - MinTemp), 0, 255);
  }
  else if ((val >= a) & (val <= c))
  {
    green = 255;
  }
  else if (val > c)
  {
    green = constrain(255.0 / (c - d) * val - (d * 255.0) / (c - d), 0, 255);
  }
  else if ((val > d) | (val < a))
  {
    green = 0;
  }

  if (val <= b)
  {
    blue = constrain(255.0 / (a - b) * val - (255.0 * b) / (a - b), 0, 255);
  }
  else if ((val > b) & (val <= d))
  {
    blue = 0;
  }
  else if (val > d)
  {
    blue = constrain(240.0 / (MaxTemp - d) * val - (d * 240.0) / (MaxTemp - d), 0, 240);
  }

  // use the displays color mapping function to get 5-6-5 color palet (R=5 bits, G=6 bits, B-5 bits)
  return M5.Lcd.color565(red, green, blue);
}
void DisplayGradient()
{
  uint8_t row, col;
  for (row = 0; row < SHARP + INSERT * (SHARP - 1); row++)
  {
    for (col = 0; col < SHARP + INSERT * (SHARP - 1); col++)
    {
      M5.Lcd.fillRect(row*3,col*3,3,3, GetColor(HDTemp[row][col]));
    }
  }
}
void setup()
{
  M5.begin();            // Initialize M5StickC Plus.  初始化 M5StickC PLus
  M5.Lcd.setTextSize(2); // Set font size.  设置字体大小
  M5.Lcd.setRotation(0); // Rotate the screen. 将屏幕旋转
  // M5.Lcd.print("Hello World");
  Wire.begin(32, 33); // sda= GPIO_21 /scl= GPIO_22
  grideye.begin();
}

void loop()
{
  for (uint8_t i = 0; i < SHARP; i++)
  {
    for (uint8_t j = 0; j < SHARP; j++)
    {
      pixels[i][j] = grideye.getPixelTemperature(i * SHARP + j);
    }
  }
  findmaxtemp();
  Getabcd();
  zoommix(); //插值
  DisplayGradient();
  M5.Lcd.setCursor(18, 170);
  M5.Lcd.print("MIN:");
  M5.Lcd.printf("%.1f", MinTemp);
  M5.Lcd.setCursor(18, 200);
  M5.Lcd.print("MAX:");
  M5.Lcd.printf("%.1f", MaxTemp);
}
